@@ -1,10 +1,11 @@
 import pandas as pd
+import numpy as np
 import datetime as dt
 import pickle
 import os
 import shutil
 import sys
-import multiprocessing
+from joblib import Parallel, delayed, cpu_count
 import subprocess
 from tqdm import tqdm
 from copy import deepcopy
@@ -753,35 +754,28 @@ class dataset(data_object):
             electrodes = em.Electrode.tolist()
 
 
-        pbar = tqdm(total = len(electrodes))
-        results = [(None, None, None)] * (max(electrodes)+1)
-        def update_pbar(ans):
-            if isinstance(ans, tuple) and ans[0] is not None:
-                results[ans[0]] = ans
-            else:
-                print('Unexpected error when clustering an electrode')
 
-            pbar.update()
-
-        spike_detectors = [clust.SpikeDetection(data_dir, x,
-                                                self.clustering_params)
-                           for x in electrodes]
         if multi_process:
-            if n_cores is None or n_cores > multiprocessing.cpu_count():
-                n_cores = multiprocessing.cpu_count() - 1
+            spike_detectors = [clust.SpikeDetection(data_dir, x,
+                                                    self.clustering_params)
+                               for x in electrodes]
 
-            pool = multiprocessing.get_context('spawn').Pool(n_cores)
-            for sd in spike_detectors:
-                pool.apply_async(sd.run, callback=update_pbar)
+            if n_cores is None or n_cores > cpu_count():
+                n_cores = cpu_count() - 1
 
-            pool.close()
-            pool.join()
+            results = Parallel(n_jobs=n_cores, verbose=10,
+                               backend='multiprocessing')(delayed(detect_spikes)
+                                                          (sd) for sd in spike_detectors)
+            # results = Parallel(n_jobs=n_cores)(delayed(run_joblib_process)(sd)
+            #                                    for sd in spike_detectors)
         else:
-            for sd in spike_detectors:
+            results = [(None, None, None)] * (max(electrodes)+1)
+            spike_detectors = [clust.SpikeDetection(data_dir, x,
+                                                    self.clustering_params)
+                               for x in electrodes]
+            for sd in tqdm(spike_detectors):
                 res = sd.run()
-                update_pbar(res)
-
-        pbar.close()
+                results[res[0]] = res
 
         print('Electrode    Result    Cutoff (s)')
         cutoffs = {}
@@ -847,14 +841,6 @@ class dataset(data_object):
             electrodes = em.Electrode.tolist()
 
 
-        pbar = tqdm(total = len(electrodes))
-        def update_pbar(ans):
-            pbar.update()
-
-        errors = []
-        def error_call(e):
-            errors.append(e)
-
         if not umap:
             clust_objs = [clust.BlechClust(self.root_dir, x, params=self.clustering_params)
                           for x in electrodes]
@@ -866,30 +852,23 @@ class dataset(data_object):
                           for x in electrodes]
 
         if multi_process:
-            if n_cores is None or n_cores > multiprocessing.cpu_count():
-                n_cores = multiprocessing.cpu_count() - 1
+            if n_cores is None or n_cores > cpu_count():
+                n_cores = cpu_count() - 1
 
-            pool = multiprocessing.get_context('spawn').Pool(n_cores)
-            for x in clust_objs:
-                pool.apply_async(x.run, callback=update_pbar, error_callback=error_call)
-
-            pool.close()
-            pool.join()
+            results = Parallel(n_jobs=n_cores, verbose=10,
+                               backend='multiprocessing')(delayed(run_joblib_process)(co)
+                                                          for co in clust_objs)
         else:
+            results = []
             for x in clust_objs:
                 res = x.run()
-                update_pbar(res)
-
-        pbar.close()
+                results.append(res)
 
         self.process_status['spike_clustering'] = True
         self.process_status['cleanup_clustering'] = False
         dio.h5io.write_electrode_map_to_h5(self.h5_file, em)
         self.save()
         print('Clustering Complete\n------------------')
-        if len(errors) > 0:
-            print('Errors encountered:')
-            print(errors)
 
     @Logger('Cleaning up clustering memory logs. Removing raw data and setting'
             'up hdf5 for unit sorting')
@@ -900,7 +879,7 @@ class dataset(data_object):
         if self.process_status['cleanup_clustering']:
             return
 
-        h5_file = dio.h5io.cleanup_clustering(self.root_dir)
+        h5_file = dio.h5io.cleanup_clustering(self.root_dir, h5_file=self.h5_file)
         self.h5_file = h5_file
         self.process_status['cleanup_clustering'] = True
         self.save()
@@ -984,7 +963,7 @@ class dataset(data_object):
             print('No unit deleted')
             return
         else:
-            tmp = dio.h5io.delete_unit(self.root_dir, unit_num)
+            tmp = dio.h5io.delete_unit(self.root_dir, unit_num, h5_file=self.h5_file)
             if tmp is False:
                 userIO.tell_user('Unit %i not found in dataset. No unit deleted'
                                  % unit_num, shell=shell)
@@ -1057,7 +1036,7 @@ class dataset(data_object):
         remove = []
         spike_count = []
         for unit in unit_table['unit_num']:
-            waves, descrip, fs = dio.h5io.get_unit_waveforms(self.root_dir, unit)
+            waves, descrip, fs = dio.h5io.get_unit_waveforms(self.root_dir, unit, h5_file=self.h5_file)
             if waves.shape[0] < min_spikes:
                 spike_count.append(waves.shape[0])
                 remove.append(unit)
@@ -1080,7 +1059,7 @@ class dataset(data_object):
             unit_name, unit_num, electrode, single_unit,
             regular_spiking, fast_spiking
         '''
-        unit_table = dio.h5io.get_unit_table(self.root_dir)
+        unit_table = dio.h5io.get_unit_table(self.root_dir, h5_file=self.h5_file)
         return unit_table
 
     def circus_clust_run(self, shell=False):
@@ -1132,6 +1111,19 @@ class dataset(data_object):
         self.make_psth_arrays()
 
 
+def detect_spikes(sd):
+    res = sd.run()
+    return res
+
+# def detect_spikes(data_dir, electrode, params, overwrite=False):
+#     sd = clust.SpikeDetection(data_dir, electrode, params=params, overwrite=overwrite)
+#     res = sd.run()
+#     return res
+
+def run_joblib_process(process):
+    res = process.run()
+    return res
+
 def port_in_dataset(rec_dir=None, shell=False):
     '''Import an existing dataset into this framework
     '''
@@ -1149,13 +1141,6 @@ def port_in_dataset(rec_dir=None, shell=False):
             print('Aborted')
             return None
 
-    # if os.path.isfile(dat.h5_file):
-    #     prompt = '%s already exists. Continuinlg will overwrite this. Continue?' % dat.h5_file
-    #     q = userIO.ask_user(prompt, shell=shell)
-    #     if q == 0:
-    #         print('Aborted')
-    #         return None
-
     if os.path.isfile(dat.log_file):
         prompt = '%s already exists. Continuing will append to this. Continue?' % dat.log_file
         q = userIO.ask_user(prompt, shell=shell)
@@ -1167,9 +1152,14 @@ def port_in_dataset(rec_dir=None, shell=False):
         print('\n==========\nPorting dataset into blechpy format\n==========\n', file=f)
         print(dat, file=f)
 
-    status = dat.process_status
-    dat.initParams(shell=shell)
+    # Check for info.rhd file or query needed info
+    info_rhd = os.path.join(dat.root_dir, 'info.rhd')
+    if os.path.isfile(info_rhd):
+        dat.initParams(shell=shell)
+    else:
+        raise FileNotFoundError(f'{info.rhd} is required for proper dataset creation') 
 
+    status = dat.process_status
 
     user_status = status.copy()
     user_status = userIO.fill_dict(user_status,
@@ -1196,8 +1186,27 @@ def port_in_dataset(rec_dir=None, shell=False):
 
     if (status['create_trial_list'] == False) and ('digital_in' in node_list):
         dat.create_trial_list()
+    else:
+        status['create_trial_list'] == True
 
     dat.save()
+
+    # Add array_time to spike_arrays/dig_in_#
+    if 'spike_trains' in node_list:
+        digs = set(x.split('.')[1] for x in node_list if 'spike_trains.' in x)
+        params = dat.spike_array_params
+        array_time = np.arange(-params['pre_stimulus'], params['post_stimulus'], 1)
+        for x in digs:
+            dio.h5io.write_array_to_hdf5(dat.h5_file, f'/spike_trains/{x}',
+                                         'array_time', array_time)
+
+        for x in dat.processing_steps:
+            status[x] = True
+            if x == 'make_unit_arrays':
+                break
+
+        dat.save()
+
     return dat
 
 
